@@ -9,9 +9,9 @@
     can be drawn
 
     Ensure that the app's icon and indicator are always drawn correctly
-    according to the size and orienation of the panel
+    according to the size and orientation of the panel
 
-    Provide visual feedback to the user when an app is lauched by pulsating
+    Provide visual feedback to the user when an app is launched by pulsating
     the app's icon
 
     Draw a highlight around the app's icon if it is the foreground app
@@ -62,9 +62,10 @@ from gi.repository import Gtk
 from gi.repository import MatePanelApplet
 from gi.repository import Gdk
 from gi.repository import Wnck
-
 from gi.repository import Gio
 from gi.repository import GObject
+from gi.repository import GLib
+
 import cairo
 import math
 import xdg.DesktopEntry as DesktopEntry
@@ -76,12 +77,14 @@ import re
 from collections import namedtuple
 
 import dock_prefs
+from docked_app_helpers import DefaultBackgroundDrawer, AlphaFillBackgroundDrawer, DefaultLightInd,\
+     DefaultDarkInd, ThemeBarInd
+
 
 from log_it import log_it as log_it
 
 ColorTup = namedtuple('ColorTup', ['r', 'g', 'b'])
 AppInfoTup = namedtuple('AppInfoTup', ['app', 'pid', 'windows'])
-ActionTup = namedtuple('ActionTup', ['name', 'command'])
 
 
 def im_get_comp_color(filename):
@@ -207,7 +210,7 @@ class PulseTimer(object):
 
     """
 
-    def __init__(self, app):
+    def __init__(self, app, once_only=False):
         """Init for the PulseTimer class.
 
         Sets everything up by creating the timer, setting a reference to the
@@ -215,11 +218,13 @@ class PulseTimer(object):
 
         Arguments:
             app : the DockedApp object
+            once_only : do only one iteration of pulsing, then stop
         """
 
         self.app = app
         self.app.pulse_step = 0
         self.app.is_pulsing = True
+        self.once_only = once_only
         self.timer_id = GObject.timeout_add(CONST_PULSE_DELAY, self.do_timer)
 
     def do_timer(self):
@@ -236,8 +241,18 @@ class PulseTimer(object):
         if self.app.pulse_step != CONST_PULSE_STEPS:
             self.app.pulse_step += 1
         else:
-            self.app.is_pulsing = False
-            GObject.source_remove(self.timer_id)
+            # if we're starting the app for the first time (with startup notification)
+            # and it still hasn't finished loading, carry on pulsing
+            if (self.app.startup_id is not None) and not self.once_only:
+                self.app.pulse_step = 0
+            else:
+                # we only want the icon to pulse one
+                # if we have a startup_id the notification needs to be cancelled
+                if self.app.startup_id is not None:
+                    self.app.cancel_startup_notification()
+
+                self.app.is_pulsing = False
+                GObject.source_remove(self.timer_id)
 
         self.app.queue_draw()
 
@@ -304,15 +319,18 @@ class DockedApp(object):
         wnck_class  : the WnckClass object relating to the app
         app_name    : e.g. Google Chrome, used for tooltips and the applet
                       right click menu etc
-        rc_actions  : list of ActionTups to hold details of right click actions
-                      the app supports
+        rc_actions  : A list of strings containing the names of the additional application
+                      actions suppported by the app
         cmd_line    : the command line and arguments used to start the app
         icon_name   : name of the app icon
         icon_filename : the filename of the app icon
         desktop_file : the filename of the app's .desktop file
+        desktop_ai   : a Gio.GDesktopAppInfo read from the .desktop file
+        startup_id   : id used for startup notifications
         wm_class_name:
         wm_class_group_name:
         applet_win  : the Gdk.Window of the panel applet
+        applet      : the panel applet
         applet_orient : the applet orientation
 
         drawing_area: Gtk.Label- provides a surface on which the app icon can
@@ -327,8 +345,10 @@ class DockedApp(object):
         is_active   : boolean - True = the app is the foreground app
         has_mouse   : boolean - True = the mouse is over the app's icon
         is_pinned   : boolean - Whether or not the app is pinned to the dock
-        indicator   : the type of indictor (light or dark) to draw under
+        indicator   : the type of indictor (e.g. light or dark) to draw under
                       running apps
+        active_bg   : the type of background (e.g. gradient or solid fill) to
+                      be drawn when the app is the active app
         ind_ws      : wnck_workspace or None - if set, indicators are to be
                       drawn for windows on the specified workspace
         last_active_win : the wnck_window of the app's last active window
@@ -361,12 +381,15 @@ class DockedApp(object):
         self.icon_name = ""
         self.icon_filename = ""
         self.desktop_file = ""
+        self.desktop_ai = None
         self.wm_class_name = ""
         self.wm_class_group_name = ""
         self.icon_geometry_set = False
         self.applet_win = None
         self.applet_orient = None
         self.ind_ws = None
+        self.startup_id = None
+        self.applet = None
 
         # all drawing is done to a Gtk.Label rather than e.g. a drawing area
         # or event box this allows panel transparency/custom backgrounds to be
@@ -396,6 +419,8 @@ class DockedApp(object):
         # set defaults
         self.indicator = dock_prefs.IndicatorType.LIGHT     # light indicator
         self.multi_ind = False                              # single indicator
+        self.active_bg = dock_prefs.ActiveBgType.GRADIENT
+
         self.last_active_win = None
 
         # set up event handler for the draw/expose event
@@ -454,7 +479,7 @@ class DockedApp(object):
             win.connect("state-changed", self.win_state_changed)
             win_class = win.get_class_group()
             win_class_name = win_class.get_res_class()
-            #if self.wm_class_group_name == "":
+            # if self.wm_class_group_name == "":
             #    self.wm_class_group_name = win.get_class_group_name()
 
             if ((win.get_window_type() == Wnck.WindowType.NORMAL) or
@@ -474,7 +499,7 @@ class DockedApp(object):
         for app in app_match_list:
             if (wnck_app.get_name() == app[0]) or (win_class_name == app[1]):
                 desktop_file = "/usr/share/applications/%s.desktop" % app[2]
-                if (os.path.exists(desktop_file)):
+                if os.path.exists(desktop_file):
                     self.desktop_file = desktop_file
                     return self.read_info_from_desktop_file()
 
@@ -483,13 +508,13 @@ class DockedApp(object):
             if not self.get_desktop_from_app_info(os.path.expanduser("~/.local/share/applications/")):
                 if not self.brute_force_find_desktop_file(os.path.expanduser("~/.local/share/applications/")):
                     if not self.get_desktop_from_app_info("/usr/share/applications/"):
-                      if not self.get_desktop_from_app_info("/usr/local/share/applications/"):
+                        if not self.get_desktop_from_app_info("/usr/local/share/applications/"):
                             if not self.brute_force_find_desktop_file("/usr/share/applications/"):
                                 if not self.brute_force_find_desktop_file("/usr/local/share/applications/"):
                                     log_it("Unable to match app")
-                                    log_it("App name: %s" %wnck_app.get_name())
-                                    log_it("Wm class: %s" %win_class_name)
-                                    log_it("cmd line: %s" %self.cmd_line)
+                                    log_it("App name: %s" % wnck_app.get_name())
+                                    log_it("Wm class: %s" % win_class_name)
+                                    log_it("cmd line: %s" % self.cmd_line)
                                     return False
 
         # now that we have the .desktop file, we can get the icon etc
@@ -725,6 +750,8 @@ class DockedApp(object):
 
         """
 
+        # TODO: replace DesktopEntry with Gio.DesktopAppInfo...
+
         # if the search dir doesn't exist, don't do anything
         if os.path.isdir(srch_dir) is False:
             return False
@@ -816,7 +843,7 @@ class DockedApp(object):
                         # they MUST match for the app to be found, and
                         # nm_part_found and exec_found become irrelevant
 
-                        if (de_wm_class.lower() == self.wm_class_name.lower()):
+                        if de_wm_class.lower() == self.wm_class_name.lower():
                             wm_class_found = True
 
                         exec_found = wm_class_found
@@ -847,6 +874,9 @@ class DockedApp(object):
             True if a match was found, False otherwise
         """
 
+        # TODO: replace DesktopEntry with Gio.DesktopAppInfo... and then
+        # TODO: remove xdg.DesktopEntry from imports and list of dependecies
+
         # if the search dir doesn't exist, don't do anything
         if os.path.isdir(srch_dir) is False:
             return False
@@ -860,8 +890,7 @@ class DockedApp(object):
                 de_exec = the_de.getExec().split(None, 1)[0]
 
                 if (self.cmd_line.find(de_exec) != -1) and \
-                   (self.wm_class_name.upper() ==
-                    the_de.getStartupWMClass().upper()):
+                   (self.wm_class_name.upper() == the_de.getStartupWMClass().upper()):
                     self.desktop_file = srch_dir+the_file
                     return True
 
@@ -883,7 +912,7 @@ class DockedApp(object):
             win_type = window.get_window_type()
             if ((win_type == Wnck.WindowType.NORMAL) or
                 (win_type == Wnck.WindowType.DIALOG)) and \
-                (window.is_skip_tasklist() == False):
+               (window.is_skip_tasklist() is False):
 
                 window.set_icon_geometry(x, y, width, height)
 
@@ -921,6 +950,16 @@ class DockedApp(object):
         """
         self.indicator = indicator
 
+    def set_active_bg (self, active_bg):
+        """Set the active background type to the value specified
+
+        Args:
+            active_bg - the background type
+
+        """
+
+        self.active_bg = active_bg
+
     def set_multi_ind(self, multi_ind):
         """ Set whether to use an indicator for each open window
 
@@ -965,9 +1004,13 @@ class DockedApp(object):
         """
 
         if self.desktop_file:
-            dfile = DesktopEntry.DesktopEntry(self.desktop_file)
-            self.app_name = dfile.getName()
-            self.icon_name = dfile.getIcon()
+            if os.path.isabs(self.desktop_file):
+                self.desktop_ai = Gio.DesktopAppInfo.new_from_filename(self.desktop_file)
+            else:
+                self.desktop_ai = Gio.DesktopAppInfo.new(self.desktop_file)
+
+            self.app_name = self.desktop_ai.get_string("Name")
+            self.icon_name = self.desktop_ai.get_string("Icon")
 
             # if the desktop file does not specify an icon name, use the app
             # name instead
@@ -979,33 +1022,12 @@ class DockedApp(object):
                 if self.icon_name == "application browser":
                     self.icon_name = "computer"
 
-            # the command specified in the .desktop should always be used to
-            # launch the app
-            self.cmd_line = dfile.getExec()
+            # get the command specified in the .desktop file used to launch the app
+            self.cmd_line = self.desktop_ai.get_string("Exec")
 
-            # now read the right click actions the app supports. These can be
-            # specified in two ways - by a key named
-            # 'X-Ayatana-Desktop-Shortcuts' or by an Actions key
-
-            xads = dfile.get('X-Ayatana-Desktop-Shortcuts')
-            if (xads is not None) and (xads != ""):
-                # the shortcut ids are in the the form of a semi-colon
-                # separated string
-                for rca in xads.split(";"):
-                    rcname = dfile.get("Name",
-                                       group="%s Shortcut Group" % rca)
-                    rcexec = dfile.get("Exec",
-                                       group="%s Shortcut Group" % rca)
-                    if (rcname != "") and (rcexec != ""):
-                        new_action = ActionTup(name=rcname, command=rcexec)
-                        self.rc_actions.append(new_action)
-
-            for action in dfile.getActions():
-                rcname = dfile.get("Name", group="Desktop Action %s" % action)
-                rcexec = dfile.get("Exec", group="Desktop Action %s" % action)
-                if (rcname != "") and (rcexec != ""):
-                    new_action = ActionTup(name=rcname, command=rcexec)
-                    self.rc_actions.append(new_action)
+            # get the list of addtional application actions (to be activated by right
+            # clicking the app's dock icon)
+            self.rc_actions = self.desktop_ai.list_actions()
 
             return True
 
@@ -1040,7 +1062,7 @@ class DockedApp(object):
             if not self.is_flashing:
                 self.is_flashing = True
                 self.flash_on = False  # initial flash state = off
-                Flasher = FlashTimer(self)
+                flasher = FlashTimer(self)
 
                 if not self.is_visible():
                     self.show_icon()
@@ -1087,37 +1109,24 @@ class DockedApp(object):
                                                self.drawing_area_size)
         ctx = cairo.Context(offscreen_surface)
 
-        # convert the highlight values to their cairo equivalents
-        red = self.highlight_color.r / 255
-        green = self.highlight_color.g / 255
-        blue = self.highlight_color.b / 255
-
         if self.is_active and (self.is_dragee is False):
-            # draw a background gradient according to the applet orientation
-            if self.applet_orient == MatePanelApplet.AppletOrient.RIGHT:
-                pattern = cairo.LinearGradient(0, 0, self.drawing_area_size, 0)
-                pattern.add_color_stop_rgba(0.0, red, green, blue, 1)
-                pattern.add_color_stop_rgba(1.0, red, green, blue, 0)
-            elif self.applet_orient == MatePanelApplet.AppletOrient.LEFT:
-                pattern = cairo.LinearGradient(self.drawing_area_size, 0, 0, 0)
-                pattern.add_color_stop_rgba(0.0, red, green, blue, 1)
-                pattern.add_color_stop_rgba(1.0, red, green, blue, 0)
-            elif self.applet_orient == MatePanelApplet.AppletOrient.DOWN:
-                pattern = cairo.LinearGradient(0, 0, 0, self.drawing_area_size)
-                pattern.add_color_stop_rgba(0.0, red, green, blue, 1)
-                pattern.add_color_stop_rgba(1.0, red, green, blue, 0)
-            else:
-                pattern = cairo.LinearGradient(0, self.drawing_area_size, 0, 0)
-                pattern.add_color_stop_rgba(0.0, red, green, blue, 1)
-                pattern.add_color_stop_rgba(1.0, red, green, blue, 0)
 
-            ctx.rectangle(0, 0, self.drawing_area_size, self.drawing_area_size)
-            ctx.set_source(pattern)
-            ctx.fill()
+            # convert the highlight values to their cairo equivalents
+            red = self.highlight_color.r / 255
+            green = self.highlight_color.g / 255
+            blue = self.highlight_color.b / 255
+
+            if self.active_bg == dock_prefs.ActiveBgType.GRADIENT:
+                dbgd = DefaultBackgroundDrawer(ctx, self.drawing_area_size,
+                                               self.applet_orient, red, green, blue)
+            else:
+                dbgd = AlphaFillBackgroundDrawer(ctx, self.drawing_area_size,
+                                                 self.applet_orient, red, green, blue, 0.5)
+
+            dbgd.draw()
 
         # draw the app icon
         Gdk.cairo_set_source_pixbuf(ctx, self.app_pb, 3, 3)
-        ctx.rectangle(0, 0, self.drawing_area_size, self.drawing_area_size)
 
         if self.is_pulsing:
             # draw the icon semi-transparently according to how far through the
@@ -1177,46 +1186,18 @@ class DockedApp(object):
                     if num_ind > 4:
                         num_ind = 4
 
-            if self.applet_orient == MatePanelApplet.AppletOrient.RIGHT:
-                ind_x = 2
-                ind_y = (self.drawing_area_size-4)/(num_ind+1) + 2
+            ind = None
+            if self.indicator == dock_prefs.IndicatorType.LIGHT:
+                ind = DefaultLightInd(ctx, self.drawing_area_size,
+                                      self.applet_orient, num_ind)
+            elif self.indicator == dock_prefs.IndicatorType.DARK:
+                ind = DefaultDarkInd(ctx, self.drawing_area_size,
+                                      self.applet_orient, num_ind)
+            elif self.indicator == dock_prefs.IndicatorType.TBAR:
+                ind = ThemeBarInd(ctx, self.drawing_area_size, self.applet_orient, self.applet)
 
-            elif self.applet_orient == MatePanelApplet.AppletOrient.LEFT:
-                ind_x = self.drawing_area_size - 1
-                ind_y = (self.drawing_area_size - 4)/(num_ind+1) + 2
-
-            elif self.applet_orient == MatePanelApplet.AppletOrient.DOWN:
-                ind_x = (self.drawing_area_size - 4)/(num_ind+1) + 2
-                ind_y = 2
-            else:
-                ind_x = (self.drawing_area_size - 4)/(num_ind+1) + 2
-                ind_y = self.drawing_area_size - 1
-
-            this_ind = 1
-            while this_ind <= num_ind:
-                rad_patt = cairo.RadialGradient(ind_x, ind_y, 2,
-                                                ind_x, ind_y, 4)
-
-                # do a light or dark indicator as necessary
-                if self.indicator == dock_prefs.IndicatorType.LIGHT:
-                    rad_patt.add_color_stop_rgba(0, 0.9, 0.9, 0.9, 1)
-                    rad_patt.add_color_stop_rgba(1, 0.0, 0.0, 0.0, 0)
-                else:
-                    rad_patt.add_color_stop_rgba(0, 0.0, 0.0, 0.0, 1)
-                    rad_patt.add_color_stop_rgba(1, 0.9, 0.9, 0.9, 0)
-
-                ctx.set_source(rad_patt)
-                ctx.arc(ind_x, ind_y, 6, 0, 2*math.pi)
-
-                if num_ind > 1:
-                    if (self.applet_orient == MatePanelApplet.AppletOrient.RIGHT) or \
-                       (self.applet_orient == MatePanelApplet.AppletOrient.LEFT):
-                        ind_y += (self.drawing_area_size - 6)/(num_ind+1)
-                    else:
-                        ind_x += (self.drawing_area_size - 6)/(num_ind+1)
-
-                this_ind += 1
-                ctx.fill()
+            if ind is not None:
+                ind.draw()
 
         # do we need a count?
         if self.show_count:
@@ -1274,7 +1255,7 @@ class DockedApp(object):
             ctx.set_font_size(fsize)
             extents = ctx.text_extents(ctext)
 
-            if (extents[3]< reqd_font_height):
+            if extents[3] < reqd_font_height:
                 font_size = fsize
                 break
 
@@ -1312,7 +1293,7 @@ class DockedApp(object):
         ctx.scale(self.drawing_area_size/draw_size,
                   self.drawing_area_size/draw_size)
 
-        ctx.move_to(left, height - 1+ adj)
+        ctx.move_to(left, height - 1 + adj)
         ctx.arc(left, inset + adj, radius, 0.5 * math.pi, 1.5*math.pi)
         ctx.arc(draw_size-inset-adj, inset + adj, radius, 1.5*math.pi, 0.5*math.pi)
         ctx.line_to(left, height - 1 + adj)
@@ -1324,7 +1305,7 @@ class DockedApp(object):
 
         # draw the text
         ctx.move_to(left - inset + width/2 - (extents[0]+extents[2]/2),
-                    (height/2) + adj +extents[3]/2)
+                    (height/2) + adj + extents[3]/2)
         ctx.set_source_rgb(tred, tgreen, tblue)
         ctx.show_text(ctext)
 
@@ -1386,7 +1367,7 @@ class DockedApp(object):
         line_width = 2          # border line width
         int_height = height - line_width * 2  # interior height
         left = 8
-        width  = draw_size - left * 2   # width of the progress bar
+        width = draw_size - left * 2   # width of the progress bar
 
         top = (draw_size/8) * 5
 
@@ -1401,7 +1382,7 @@ class DockedApp(object):
 
         # fill the interior with the background colour
         ctx.set_line_width(1)
-        ctx.set_source_rgb (bk_red, bk_green, bk_blue)
+        ctx.set_source_rgb(bk_red, bk_green, bk_blue)
         rounded_rectangle(ctx, left + line_width, top + line_width,
                           width - line_width*2, height-line_width*2,
                           2)
@@ -1428,9 +1409,12 @@ class DockedApp(object):
 
     def start_app(self):
         """Start the app or open a new window if it's already running
+
+            Use Gio.DesktopAppinfo as it supports startup notfication
         """
-        the_de = DesktopEntry.DesktopEntry(self.desktop_file)
-        run_it = the_de.getExec()
+        # start the app
+        run_it = self.desktop_ai.get_string("Exec")
+
         if run_it is not None:
 
             # hack for Linux Mint:
@@ -1447,30 +1431,75 @@ class DockedApp(object):
             # expecting....
             if run_it == "/usr/bin/startcaja":
                 run_it = "caja"
+                self.run_cmd_line(run_it)
+                return
 
-            # remove any command line arguments beginning with ' %'
-            # TODO - check if still really necessary
-            if " %" in run_it:
-                i = run_it.rfind(" %")
-                run_it = run_it[0:i]
+        gdai = Gio.DesktopAppInfo.new_from_filename(self.desktop_file)
+        disp = Gdk.Display.get_default()
+        if build_gtk2:
+            alc = Gdk.AppLaunchContext()
+        else:
+            alc = disp.get_app_launch_context()
+    
+        alc.set_desktop(-1)  # use default screen & desktop
+        alc.set_timestamp(Gtk.get_current_event_time())
+        alc.connect("launch-failed", self.launch_failed)
 
-        # start the app
-        self.run_cmd_line(run_it)
+        # indicate we want startup notification
+        self.startup_id = alc.get_startup_notify_id(gdai, [])
+
+        gdai.launch_uris_as_manager([], alc, GLib.SpawnFlags.SEARCH_PATH,
+                                    None, None, None, None)
+
+        # make the app's icon pulse
+        if not self.is_running():
+            throbber = PulseTimer(self)
+        else:
+            # if the app is already running, we want the icon to pulse at most once only
+            # For apps which don't open a new window, the pulse timer will end up cancelling
+            # the unneeded startup notification
+            throbber = PulseTimer(self, True)
+
+    def cancel_startup_notification(self):
+        """
+            Cancel any startup notification
+        """
+        if build_gtk2:
+            Gdk.notify_startup_complete_with_id(self.startup_id)
+        else:
+            display = Gdk.Display.get_default()
+            display.notify_startup_complete(self.startup_id)
+           
+        self.startup_id = None
+
+    def launch_failed(self, app_launch_context, startup_id):
+        """Handler for app launch failure events
+
+        Cancel the satartup notification
+
+            Args:
+                app_launch_context : the Gdk.AppLaunchContext that failed
+                startup_id : the startup notification id
+        """
+
+        self.cancel_startup_notification()
+
+        display = Gdk.Display.get_default()
+        display.notify_startup_complete(startup_id)
 
     def run_cmd_line(self, cmd_line):
-        """Run a command line. To be used when starting an app for the first
-           time or when running an action/shortcut specfied in the app's
-           .desktop file
+        """Run a command line.
 
             Args:
                 cmd_line - the command to run
         """
 
+        # TODO: this is old code and needs to be removed
+
         # the command line may contain escape sequences, so unescape them....
         cmd_line = bytearray(cmd_line, "UTF-8")
         cmd_line = cmd_line.decode("unicode-escape")
 
-
         # if an environment variable is specified, extract its name an value
         # Note: the .desktop file specification at
         # https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s06.html
@@ -1505,15 +1534,20 @@ class DockedApp(object):
             cmd_line = head + "/" + tail
         app_info = Gio.AppInfo.create_from_commandline(cmd_line,
                                                        None,
-                                                       Gio.AppInfoCreateFlags.NONE)
+                                                       Gio.AppInfoCreateFlags.SUPPORTS_STARTUP_NOTIFICATION)
         alc = Gdk.AppLaunchContext()
         alc.set_desktop(-1)                   # use default screen & desktop
+        alc.set_timestamp(Gtk.get_current_event_time())
+
         # if the .desktop specfied an environment variable, set it
         if (var_name is not None) and (var_value is not None):
             alc.setenv(var_name, var_value)
 
-        app_info.launch(None, alc)
-        # set the app icon pulsing
+        file_list = GLib.List()
+
+        # app_info.launch(None, alc)
+        self.startup_id = alc.get_startup_notify_id(app_info, [])
+
         throbber = PulseTimer(self)
 
     def run_rc_action(self, act_no):
@@ -1524,7 +1558,21 @@ class DockedApp(object):
         """
 
         if len(self.rc_actions) >= act_no:
-            self.run_cmd_line(self.rc_actions[act_no-1].command)
+            if build_gtk2:
+                alc = Gdk.AppLaunchContext()
+            else:
+                disp = Gdk.Display.get_default()
+                alc = disp.get_app_launch_context()
+
+            alc.set_desktop(-1)  # use default screen & desktop
+            alc.set_timestamp(Gtk.get_current_event_time())
+            alc.connect("launch-failed", self.launch_failed)
+
+            # indicate we want startup notification
+            self.startup_id = alc.get_startup_notify_id(self.desktop_ai, [])
+
+            self.desktop_ai.launch_action(self.rc_actions[act_no-1], alc)
+            self.start_pulsing()
 
     def get_rc_action(self, act_no):
         """ return a specified right click action's details
@@ -1536,14 +1584,12 @@ class DockedApp(object):
                 bool - True if the action exists, False otherwise
                 string - the name of the action (i.e. the text to appear in the
                 right click menu)
-                string - the command line to the be run
         """
 
         if len(self.rc_actions) >= act_no:
-            return (True, self.rc_actions[act_no-1].name,
-                    self.rc_actions[act_no-1].command)
+            return True, self.desktop_ai.get_action_name(self.rc_actions[act_no-1])
         else:
-            return (False, "", "")
+            return False, ""
 
     def start_pulsing(self):
         """ start the dock icon pulsing
@@ -1570,7 +1616,7 @@ class DockedApp(object):
             is_visible : whether the progress is to be displayed
         """
 
-        if (is_visible != self.show_progress):
+        if is_visible != self.show_progress:
             self.show_progress = bool(is_visible)
             self.queue_draw()
 
